"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var React = require("react");
var rotateTimer = 0, angleNum = 0;
var audio;
var AudioPlayer = function (props, ref) {
    var info = props.info, refs = props.refs, _a = props.autoplay, autoplay = _a === void 0 ? false : _a, children = props.children, onChange = props.onChange, onPlaying = props.onPlaying, onOperate = props.onOperate, onError = props.onError, _b = props.curTime, curTime = _b === void 0 ? 0 : _b, onTimeUpdate = props.onTimeUpdate;
    var _c = react_1.useState(autoplay), playing = _c[0], setPlaying = _c[1];
    var _d = react_1.useState(false), mouse = _d[0], setMouse = _d[1];
    var _e = react_1.useState(info[0]), currentMusic = _e[0], setCurrentMusic = _e[1];
    var audioRef = react_1.useRef(null);
    react_1.useEffect(function () {
        audio = audioRef && audioRef.current;
        if (audio) {
            audio.addEventListener('canplay', audioCanplay);
            audio.addEventListener('error', audioError);
            //设置初始音量
            if (refs.volumeProgress) {
                refs.volumeProgress.current.style.width = "50%";
            }
            audio.volume = 0.5;
            audio.addEventListener('loadedmetadata', function () {
                audio.addEventListener('timeupdate', timeUpdate);
                audio.currentTime = curTime;
            });
        }
    }, []);
    react_1.useEffect(function () {
        if (playing) {
            setTimeout(function () {
                play();
            }, 1000);
        }
        return function () {
            clearInterval(rotateTimer);
            audio.removeEventListener('canplay', audioCanplay);
            audio.removeEventListener('error', audioError);
            audio.removeEventListener('timeupdate', timeUpdate);
            audio = null;
        };
    }, [currentMusic.src]);
    react_1.useEffect(function () {
        if (onPlaying && typeof onPlaying === 'function') {
            onPlaying(playing);
        }
        if (playing) {
            avatarTransform();
        }
        else {
            clearInterval(rotateTimer);
        }
    }, [playing]);
    // 播放
    var play = function () {
        if (audio.paused && currentMusic.src) {
            var playPromise = audio.play();
            if (playPromise !== undefined) {
                playPromise
                    .then(function () {
                    setPlaying(true);
                })
                    .catch(function () {
                    setPlaying(false);
                    audio.pause();
                });
            }
        }
        else {
            setPlaying(false);
            audio.pause();
        }
    };
    // 播放海报动画
    var avatarTransform = function () {
        if (refs.musicAvatar.current) {
            rotateTimer = setInterval(function () {
                angleNum = angleNum + 1;
                refs.musicAvatar.current.style.transform = "rotate(" + angleNum + "deg)";
            }, 70);
        }
    };
    // 监听播放总时间
    var audioCanplay = function () {
        var totalTime = parseInt(audio.duration);
        if (refs.totalTime.current) {
            refs.totalTime.current.innerHTML = getTime(totalTime);
        }
        if (refs.remainTime.current) {
            refs.remainTime.current.innerHTML = getTime(0);
        }
    };
    // 监听播放失败
    var audioError = function () {
        console.log("播放失败");
        if (onError && typeof onError === 'function') {
            onError('播放失败');
        }
    };
    // 监听音频时间
    var timeUpdate = function () {
        //设置播放进度条
        var playPer = audio.currentTime / audio.duration;
        if (refs.played.current) {
            refs.played.current.style.width = playPer * 100 + "%";
        }
        if (!audio.paused && onTimeUpdate && typeof onTimeUpdate === 'function') {
            onTimeUpdate(audio.currentTime, audio.duration);
        }
        //设置缓冲进度条
        var timeRages = audio.buffered;
        var bufferedTime = 0;
        if (timeRages.length !== 0) {
            bufferedTime = timeRages.end(timeRages.length - 1);
        }
        var bufferedPer = bufferedTime / audio.duration;
        if (refs.buffered.current) {
            refs.buffered.current.style.width = bufferedPer * 100 + "%";
        }
        //设置剩余时间
        var remainTime = parseInt(audio.currentTime);
        if (refs.remainTime.current) {
            refs.remainTime.current.innerHTML = getTime(remainTime);
        }
        if (audio.ended) {
            // next()
            setPlaying(false);
            if (onOperate && typeof onOperate === 'function') {
                var v = {
                    option: 'end',
                    music: currentMusic
                };
                return onOperate(v);
            }
        }
    };
    // 播放下一个
    var next = function () {
        angleNum = 0;
        if (!currentMusic.src) {
            return;
        }
        var current = 0;
        info.forEach(function (v, i) {
            if (v.src === currentMusic.src) {
                current = i;
            }
        });
        if (current === info.length - 1) {
            if (audio.ended) {
                audio.pause();
                setPlaying(false);
            }
            if (onOperate && typeof onOperate === 'function') {
                var v = {
                    option: 'next',
                    music: currentMusic
                };
                return onOperate(v);
            }
        }
        var data = info[current < info.length - 1 ? current + 1 : 0];
        setCurrentMusic(data);
        if (onChange && typeof onChange === 'function') {
            onChange(data);
        }
    };
    // 播放上一个
    var last = function () {
        angleNum = 0;
        if (!currentMusic.src) {
            return;
        }
        var current = 0;
        info.forEach(function (v, i) {
            if (v.src === currentMusic.src) {
                current = i;
            }
        });
        if (current === 0) {
            if (onOperate && typeof onOperate === 'function') {
                var v = {
                    option: 'prev',
                    music: currentMusic
                };
                return onOperate(v);
            }
        }
        var data = info[current > 0 ? current - 1 : info.length - 1];
        setCurrentMusic(data);
        if (onChange && typeof onChange === 'function') {
            onChange(data);
        }
    };
    // 快进快退15s
    var fastForward = function (time) {
        if (audio.currentTime !== 0) {
            audio.currentTime = audio.currentTime + time;
            if (audio.paused) {
                audio.play();
                setPlaying(true);
            }
        }
    };
    // 获取时间
    var getTime = function (musicTime) {
        if (musicTime) {
            if (musicTime < 60) {
                return "00:" + (musicTime < 10 ? "0" + musicTime : musicTime);
            }
            else {
                var time = parseInt("" + musicTime / 60);
                return (time < 10 ? "0" + time : parseInt(time)) + ":" + (musicTime % 60 < 10 ? "0" + musicTime % 60 : musicTime % 60);
            }
        }
        else {
            return "00:00";
        }
    };
    // 设置时间
    var setTime = function (e) {
        if (audio.currentTime !== 0 && refs.progress) {
            var pageX = !e.pageX ? e.touches[0].pageX : e.pageX;
            if (pageX - refs.played.current.getBoundingClientRect().left <= refs.progress.current.offsetWidth && pageX - refs.played.current.getBoundingClientRect().left > 0) {
                var newWidth = (pageX - refs.played.current.getBoundingClientRect().left) / refs.progress.current.offsetWidth;
                refs.played.current.style.width = newWidth * 100 + "%";
                if (!mouse) {
                    audio.currentTime = newWidth * audio.duration;
                }
            }
            if (audio.paused) {
                audio.play();
                setPlaying(true);
            }
        }
    };
    // 设置音量
    var setVolume = function (e) {
        var volumeRate = (!e.pageX ? e.touches[0].pageX : e.pageX - refs.totalVolume.current.getBoundingClientRect().left) / refs.totalVolume.current.offsetWidth;
        if (volumeRate > 0.01 && volumeRate <= 1) {
            audio.volume = volumeRate;
            if (refs.volumeProgress) {
                refs.volumeProgress.current.style.width = volumeRate * 100 + "%";
            }
        }
        else if (volumeRate <= 0.01) {
            audio.volume = 0;
        }
        else {
            audio.volume = 1;
        }
    };
    // 改变时间
    var changeTime = function (e) {
        if (audio.currentTime !== 0) {
            setTime(e);
        }
    };
    // 滑动时间
    var slideTime = function (e) {
        if (mouse && audio.currentTime !== 0) {
            setTime(e);
        }
    };
    // 改变音量
    var changeVolume = function (e) {
        if (audio.currentTime !== 0) {
            setVolume(e);
        }
    };
    // 滑动音量
    var slideVolume = function (e) {
        if (mouse && audio.currentTime !== 0) {
            setVolume(e);
        }
    };
    // 鼠标离开
    var mouseLeave = function () {
        setMouse(false);
    };
    // 鼠标按下
    var mouseDown = function () {
        setMouse(true);
    };
    // 播放指定音乐
    var playMusic = function (id) {
        var music = info.filter(function (v) { return v.id === id; });
        if (music.length > 0) {
            setPlaying(true);
            setCurrentMusic(music[0]);
        }
        else {
            console.warn('没有此音乐');
        }
    };
    // 当前播放时间
    var onTime = function () {
        if (audio && audio.currentTime !== 0) {
            return audio.currentTime;
        }
        return 0;
    };
    // 当前播放状态
    var isPlay = function () {
        return playing || false;
    };
    // 开放Api
    react_1.useImperativeHandle(ref, function () { return ({
        playMusic: playMusic,
        play: play,
        next: next,
        last: last,
        mouseLeave: mouseLeave,
        mouseDown: mouseDown,
        slideTime: slideTime,
        changeTime: changeTime,
        slideVolume: slideVolume,
        changeVolume: changeVolume,
        fastForward: fastForward,
        onTime: onTime,
        isPlay: isPlay,
    }); });
    return (React.createElement("div", null,
        children,
        React.createElement("audio", { src: currentMusic.src ? currentMusic.src : "", ref: audioRef })));
};
exports.default = react_1.forwardRef(AudioPlayer);
