import { useState, useEffect, useRef, useImperativeHandle, forwardRef, createRef } from 'react'
import * as React from 'react';
import { message } from 'antd';
import AudioPlayer from '../../player'
import './style.css';

export interface IMusic {
  img?: string;
  id: string;
  artist?: string;
  name?: string;
  src: string;
}
export interface IOperate {
  option: string;
  music: IMusic;
}

export interface PlayProps {
  info: IMusic[]
  src: any
  autoplay: boolean
  poster: string
  onChange: (arg: IMusic) => void
  onOperate: (arg: IOperate) => void
}

const MAudio: React.SFC<PlayProps> = (props, ref) => {
  const {
    info,
    src,
    poster,
    autoplay,
    onChange,
    onOperate,
  } = props
  const [playing, setPlaying] = useState(false)
  const [music, setMusic] = useState<IMusic>()
  let musicRef: any = createRef();
  let refs: any = {
    musicAvatar: useRef(),
    progress: useRef(),
    buffered: useRef(),
    played: useRef(),
    totalTime: useRef(),
    remainTime: useRef(),
    totalVolume: useRef(),
    volumeProgress: useRef()
  }

  // 开放Api
  useImperativeHandle(ref, () => ({
    onPlay: (id: any) => {
      musicRef.current.playMusic(id)
    }
  }))

  useEffect(() => {
    setMusic(info[0])
  }, [])


  const handleMusic = (method: string, time?: number) => {
    musicRef.current[method](time)
  }

  const onMusicEvents = (e: any, method: string) => {
    musicRef.current[method](e)
  }

  const hanleChange = (value: IMusic) => {
    setMusic(value)
    if (onChange && typeof onChange === 'function') {
      onChange(value)
    }
  }

  const hanleOption = (value: IOperate) => {
    if (onOperate && typeof onOperate === 'function') {
      onOperate(value)
    }
  }
  console.log(playing)
  return (
    <AudioPlayer 
      info={info} 
      ref={musicRef} 
      refs={refs} 
      autoplay={autoplay} 
      onPlaying={(value: boolean) => setPlaying(value)}
      onChange={(value: IMusic) => hanleChange(value)} 
      onOperate={(value: IOperate) => hanleOption(value)} 
      onError={error => message.error(error)}
      >
      <div className='audioPanel'>
        {/* <div className={.tool}>
            <div className='video-on'  onClick={{(value: IMusic) => hanleChange(value)}}></div>
            <div  className={rateClass} onClick={handleRate}></div>
          </div> */}
        <div className='control'>
          <img
            className='icon_btn'
            src={'./images/ic_prev_time.png'}
            onClick={() => handleMusic('fastForward', -15)}
          />
          <img
            className='icon_btn'
            src={'./images/ic_prev.png'}
            onClick={() => handleMusic('last')}
          />
          <div>
            <img className='play' src={!playing ? './images/ic_play.png' : './images/ic_pause.png'} onClick={() => handleMusic('play')} />
          </div>
          <img
            className='icon_btn'
            src={'./images/ic_next.png'}
            onClick={() => handleMusic('next')}
          />
          <img
            className='icon_btn'
            src={'./images/ic_next_time.png'}
            onClick={() => handleMusic('fastForward', 15)}
          />
        </div>
        <div className='audiobar'>
          <img
            className='icon_play'
            src={!playing ? './images/icon_play.png' : './images/icon_pause.png'}
            onClick={() => handleMusic('play')}
          />
          <div className='startime' ref={refs.remainTime}>00:00</div>

          <div className='progress-wrapper' ref={refs.progress}
            onTouchMove={(e) => onMusicEvents(e, 'changeTime')}
            onTouchStart={(e) => onMusicEvents(e, 'changeTime')}
            onClick={(e) => onMusicEvents(e, 'changeTime')}
            onMouseDown={(e) => onMusicEvents(e, 'mouseDown')}
            onMouseMove={(e) => onMusicEvents(e, 'slideTime')}
            onMouseUp={(e) => onMusicEvents(e, 'mouseLeave')}
            onMouseLeave={(e) => onMusicEvents(e, 'mouseLeave')}
          >
            <div className='progress'>
              <div className='progress-buffered' ref={refs.buffered}></div>
              <div className='progress-played' ref={refs.played}></div>
            </div>
          </div>

          <div className='endtime' ref={refs.totalTime}>00:00</div>
          <div className='volume-control' ref={refs.totalVolume}>
            <div className='volume-progress' ref={refs.volumeProgress}></div>
          </div>
        </div>
      </div>
    </AudioPlayer>
  )
}

export default forwardRef(MAudio)
